import { defineStore } from 'pinia'
import useSpentTime from '@/hooks/useSpentTime'

interface MarkDelayItem {
  startScore: number
  endScore: number
  minMarkTime: number
}
interface MarkStoreState {
  // 前端评卷试题的分值
  curQuestionScore: number | null
  // 倒计时相关状态
  countdownTime: number
  countdownTimer: NodeJS.Timeout | null
  // 评卷延时配置
  markDelay: MarkDelayItem[]
  markSpent: {
    startTime: number
    endTime: number
  }
}

interface MarkStoreActions {
  setCurQuestionScore: (val: number | null) => void
  startCountdown: (markSpeedLimit?: number) => void
  clearCountdown: () => void
  updateCountdownTime: (time: number) => void
  checkNeedCountDown: (score: number) => void
  initMarkSpent: () => void
  setMarkDelay: (val: string) => void
  getMarkLevelSpeedLimit: (score: number) => number
}

const useMarkStore = defineStore<'mark', MarkStoreState, Record<string, any>, MarkStoreActions>('mark', {
  state() {
    return {
      curQuestionScore: null,
      countdownTime: 0,
      countdownTimer: null,
      markDelay: [],
      markSpent: {
        startTime: 0,
        endTime: 0,
      },
    }
  },
  getters: {
    isCountingDown: (state: MarkStoreState) => state.countdownTime > 0,
    buttonText: (state: MarkStoreState) => (state.countdownTime > 0 ? `${state.countdownTime}s` : '确定'),
  },
  actions: {
    initMarkSpent() {
      this.markSpent = {
        startTime: Date.now(),
        endTime: 0,
      }
    },
    setMarkDelay(val: string) {
      const markDelay = val ? (JSON.parse(val) as MarkDelayItem[]) : []
      this.markDelay = markDelay
    },
    getMarkLevelSpeedLimit(score: number): number {
      if (this.markDelay.length) {
        const speedLimit = this.markDelay.find((limit) => {
          return limit.endScore >= score && limit.startScore <= score
        })
        return speedLimit?.minMarkTime || 0
      } else {
        return 0
      }
    },
    checkNeedCountDown(score: number) {
      if (!this.markSpent.startTime) return
      this.markSpent.endTime = Date.now()
      const markLevelSpeedLimit = this.getMarkLevelSpeedLimit(score)
      const waitTime = markLevelSpeedLimit - Math.round((this.markSpent.endTime - this.markSpent.startTime) / 1000)
      this.startCountdown(Math.max(waitTime, 0))
    },

    setCurQuestionScore(val: number | null) {
      this.curQuestionScore = val
    },

    startCountdown(markSpeedLimit?: number) {
      // 清除之前的计时器
      this.clearCountdown()

      if (!markSpeedLimit || markSpeedLimit <= 0) {
        this.countdownTime = 0
        return
      }

      this.countdownTime = markSpeedLimit
      this.countdownTimer = setInterval(() => {
        this.countdownTime--
        if (this.countdownTime <= 0) {
          this.clearCountdown()
        }
      }, 1000)
    },

    clearCountdown() {
      if (this.countdownTimer) {
        clearInterval(this.countdownTimer)
        this.countdownTimer = null
      }
      this.countdownTime = 0
    },

    updateCountdownTime(time: number) {
      this.countdownTime = time
    },
  },
})

export default useMarkStore
